import json
import requests
from aprsfi.response import Response


class RequestException(Exception):
    pass


class API(object):

    def __init__(self, api_key):
        self._api_key = api_key
        self._api_url = ("http://api.aprs.fi/api/get?name={name}&what={what}&"
                         "apikey=%s&format=json" % api_key)

    def __getattr__(self, attr):
        if attr in ['loc', 'wx']:
            def request(*args):
                url = self._api_url.format(name=','.join(args), what=attr)
                response = requests.get(url)
                response_json = response.json()
                if response_json['result'] == 'fail':
                    raise RequestException(
                        response_json['description'])
                return Response(response_json)
            return request
        else:
            raise AttributeError



class Entry(object):

    def __init__(self, data):
        self._data = data

    @property
    def fields(self):
        return list(self._data)

    def __eq__(self, another_entry):
        return self._data == another_entry._data

    def __getattr__(self, attr):
        if attr in self._data:
            return self._data[attr]
        else:
            raise AttributeError



from aprsfi.entry import Entry


class Response(object):

    def __init__(self, response_dict):
        request_entries = response_dict.pop('entries', None)
        if request_entries:
            self.entries = [Entry(entry) for entry in request_entries]
        self._data = response_dict

    def __eq__(self, another_response):
        self_response_names = list(map(lambda e: e.name, self.entries))
        self_response_names.sort()
        another_response_names = list(map(lambda e: e.name,
                                          another_response.entries))
        another_response_names.sort()
        return (self._data == another_response._data and
                self_response_names == another_response_names)

    def __getattr__(self, attr):
        if attr in self._data:
            return self._data[attr]
        else:
            raise AttributeError



#!/usr/bin/env python

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name='aprsfi',
    version='0.2',
    license='LGPLv3',
    description='python client aprs.fi API.',
    long_description='A complete Python client for aprs.fi',
    packages=['aprsfi'],
    author="Whanderley Souza Freitas",
    author_email='whanderley.souza@iff.edu.br',
    install_requires=['requests'],
    url='https://github.com/SAEG-DSE/aprsfi',
    download_url='https://github.com/SAEG-DSE/python-aprsfi/archive/0.2.tar.gz',
    keywords='aprsfi api client wrapper',
)
